
# ============================================================
# MainWindow.py
# main window for the case management program.
# ------------------------------------------------------------
# Copyright (C) 2005 Juergen Gerner.
# ============================================================

import kumula
from qt import *
from SideBar import KSideBar
from KumulaMainWindow import KKumulaMainWindow

import sys
import os

# ------------------------------------------------------------


class KMainWindow(KKumulaMainWindow):

    def __init__(self, aCaseID):
        self.CaseID = aCaseID
        KKumulaMainWindow.__init__(self)


    def getProgramName(self):
        return "CaseData"

    def getProgramVersion(self):
        return "0.1"

    def getProgramCopyright(self):
        return "Copyright (C) 2005-2006 Juergen Gerner."


    def createUI(self):
        KKumulaMainWindow.createUI(self)
        # initalize user interface
        self.setCentralWidget(QWidget(self, "qt_central_widget"))
        myLayout = QHBoxLayout(self.centralWidget())

        self.folderView = KSideBar(self.centralWidget(), "folderView")
        myLayout.addWidget(self.folderView)

        self.widgetStack = QWidgetStack(self.centralWidget())

        self.loadWidgets()
        myLayout.addWidget(self.widgetStack)

        self.resize(QSize(1,1).expandedTo(self.minimumSizeHint()))
        self.clearWState(Qt.WState_Polished)



    def createMenuBar(self):
        pass
# no, we don't need a MenuBar here...
#        KKumulaMainWindow.createMenuBar(self)



    def createToolBar(self):
        pass
# no, we don't need a ToolBar here...
#        KKumulaMainWindow.createToolBar(self)


    def __tr(self,s,c = None):
        return qApp.translate("MainWindow",s,c)


    def languageChange(self):
        self.updateCaption()


    def updateView(self):
        self.widgetStack.raiseWidget(self.folderView.currentButton())



    def handleCaseClose(self):
        kumula.Program.closeAllWindows()



    def loadWidgets(self):
        self.widgetByID = {}
        firstID = -99

        myViewerDir = os.path.join(os.path.dirname(sys.argv[0]), "..", "lib", "casedata")
        # get list of all "widget_" files in KUMULADIR/lib/casedata
        osList = os.listdir(myViewerDir)
        osList.sort()
        for name in osList:
            myFileName = QString(name)
            if (myFileName.left(7) == "widget_"):
                myFile = os.path.join(myViewerDir, name)

                myPlugin = open(myFile, "r")
                # ...compile it...
                exec myPlugin in locals()
                myPlugin.close()
                # ...and add it to our tab bar
                myWidget = CaseWidget(self.widgetStack)
                myWidget.setCaseID(self.CaseID)
                myID = self.widgetStack.addWidget(myWidget)
                if (firstID == -99):
                    firstID = myID

                myTitle = myWidget.caption()
                self.folderView.insertButton(myTitle, myWidget.icon(), myID)
                if (myID > 0) and (self.CaseID == 0):
                    self.folderView.getButton(myID).setEnabled(0)
                self.widgetByID[myID] = myWidget

        self.folderView.insertSpacer()
        self.connect(self.folderView, PYSIGNAL("selectionChanged()"), self.updateView)
        self.folderView.selectButton(firstID)

        self.connect(self.widgetByID[firstID], PYSIGNAL("caseChanged()"), self.caseChanged)


    def caseChanged(self, aCaseID):
        self.folderView.enableAllButtons()
        self.CaseID = aCaseID
        self.updateCaption()
        for myWidgetID in self.widgetByID:
            self.widgetByID[myWidgetID].setCaseID(aCaseID)


    def updateCaption(self):
        self.CaseTitle = ""
        if (self.CaseID == 0):
            self.CaseTitle = "New case"
        else:
            quCase = kumula.DB.doSelect("cases", ("title",), "", {"caseid":"%i" % self.CaseID})
            while quCase.next():
                self.CaseTitle = quCase.value(0).toString()

        # because CaseData is an "internal" application,
        # I don't want the "KUMULA" in the window caption.
        # So I call QMainWindow.setCaption rather than self./KKumulaMainWindow.setCaption!
        QMainWindow.setCaption(self, self.CaseTitle)

