

from widgetTmpl_header import WidgetHeader
import kumula
from qt import *
import UserInterface


class CaseWidget(WidgetHeader):

    def __init__(self, parent):
        WidgetHeader.__init__(self, parent)
        self.setIcon(kumula.getSideBarIcon("header"))
        self.btnSuggestNumber.setText("")
        self.btnSuggestNumber.setPixmap(kumula.getMenuBarIcon("suggest_number"))


    def setCaseID(self, aCaseID):
        self.CaseID = aCaseID
        if (self.CaseID == 0):
            # self.btnCreateCase.show()
            self.btnCreateCase.setText("Create case")
            self.initTypeList()
            self.initUserList()
            self.suggestNumber()
        else:
            # self.btnCreateCase.hide()
            self.btnCreateCase.setText("Update case data")
            # load values for this case
            quCaseData = kumula.DB.doSelect("cases", ("reference","title","casetype","userid","comment","archived"), "", {"caseid":"%i" % self.CaseID})
            while quCaseData.next():
                myReference = quCaseData.value(0).toString()
                myTitle     = quCaseData.value(1).toString()
                myType      = quCaseData.value(2).toInt()
                myUserID    = quCaseData.value(3).toInt()
                myComment   = quCaseData.value(4).toString()
                myArchived  = quCaseData.value(5).toInt()
                self.edReference.setText(myReference)
                self.edTitle.setText(myTitle)
                self.initTypeList(myType)
                self.initUserList(UserInterface.getUserByID(myUserID).userName)
                self.edComment.setText(myComment)
                if (myArchived > 0):
                    self.btnCreateCase.setEnabled(0)


    def suggestNumber(self):
        myStatement = QString("SELECT MAX(caseid) FROM cases;")
        myQuery = kumula.DB.execStatement(myStatement)
        freeNr = 1
        if (myQuery.next()):
            freeNr = myQuery.value(0).toInt() + 1
        mySuggestion = QString("%i-" % QDate.currentDate().year())
        mySuggestion.append("%i/" % freeNr)
        mySuggestion.append(self.getSelectedUser())
        self.edReference.setText(mySuggestion)
        self.edReference.setFocus()



    def checkInputValues(self):
        myReference = self.edReference.text().stripWhiteSpace()
        if (myReference.length() == 0):
            QMessageBox.critical(self, "Error", "You have to enter a<br>valid reference for this case.<br><br>I suggest a free one<br>for you...")
            self.suggestNumber()
            self.edReference.setFocus()
            return 0
        if (self.CaseID == 0):
            cntNr = kumula.DB.doCount("cases", {"reference":myReference})
            if (cntNr > 0):
                QMessageBox.information(self, "Error", "This reference is already in use<br>for another case.<br><br>I suggest a free one<br>for you...")
                self.suggestNumber()
                self.edReference.setFocus()
                return 0
        myTitle = self.edTitle.text().stripWhiteSpace()
        if (myTitle.length() == 0):
            QMessageBox.critical(self, "Error", "You have to enter a<br>valid title for this case.")
            self.edTitle.setFocus()
            return 0
        return 1


    def createCase(self):
        if (self.checkInputValues() > 0):
            myReference = self.edReference.text()
            myTitle = self.edTitle.text()
            myType = "%i" % self.getSelectedType()
            myUserID = QString("%i" % UserInterface.KUser(self.getSelectedUser()).userID)
            myComment = self.edComment.text()
            if (self.CaseID == 0):
                kumula.DB.doInsert("cases", {"reference":myReference,
                                             "title":myTitle,
                                             "casetype":myType,
                                             "userid":myUserID,
                                             "comment":myComment,
                                             "date_creation":QDate.currentDate().toString(Qt.ISODate),
                                             "archived":"0"})
                myCaseID = kumula.DB.lastAutoID()
                self.setCaseID(myCaseID)
            else:
                kumula.DB.doUpdate("cases", {"reference":myReference,
                                             "title":myTitle,
                                             "casetype":myType,
                                             "userid":myUserID,
                                             "comment":myComment}, {"caseid":"%i" % self.CaseID})
            self.emit(PYSIGNAL("caseChanged()"), (self.CaseID,))



    def initTypeList(self, aType=0):
        self.cbType.clear()
        quTypes = kumula.DB.doSelect("cases_casetypes", ("id","title"), "title")
        while quTypes.next():
            myID    = quTypes.value(0).toInt()
            myTitle = quTypes.value(1).toString()
            myEntry = QString(myTitle)
            myEntry.append(' (')
            myEntry.append("%i" % myID)
            myEntry.append(')')
            self.cbType.insertItem(myEntry)
            if (myID == aType):
                self.cbType.setCurrentText(myEntry)


    def getSelectedType(self):
        myEntry = self.cbType.currentText()
        startPos = myEntry.findRev('(')
        myType = myEntry.mid(startPos+1, myEntry.length()-startPos-2).toInt()[0]
        return myType


    def initUserList(self, anUserName=''):
        preUserName = anUserName
        if (preUserName == ""):
            preUserName = kumula.UserName
        self.cbUser.clear()
        allUsers = UserInterface.getAllUsers()
        for user in allUsers:
            myEntry = QString(user.getFormattedName())
            myEntry.append(' (')
            myEntry.append(user.userName)
            myEntry.append(')')
            self.cbUser.insertItem(myEntry)
            if (user.userName == preUserName):
                self.cbUser.setCurrentText(myEntry)

    def getSelectedUser(self):
        myEntry = self.cbUser.currentText()
        startPos = myEntry.findRev('(')
        myUserName = myEntry.mid(startPos+1, myEntry.length()-startPos-2)
        return myUserName

