
from widgetTmpl_parties import WidgetParties

import kumula
from qt import *
from ClientSelectDlg import KClientSelectDlg
import ClientInterface


class CaseWidget(WidgetParties):

    def __init__(self, parent):
        WidgetParties.__init__(self, parent)
        self.setIcon(kumula.getSideBarIcon("parties"))


    def setCaseID(self, aCaseID):
        self.CaseID = aCaseID
        if (self.CaseID > 0):
            quCaseData = kumula.DB.doSelect("cases", ("archived",), "caseid", {"caseid":"%i" % self.CaseID})
            while quCaseData.next():
                myArchived  = quCaseData.value(0).toInt()
                if (myArchived > 0):
                    self.btnAddClient.setEnabled(0)
                    self.btnRemoveClient.setEnabled(0)
                    self.btnUpdateClientInfo.setEnabled(0)
                    self.btnAddContrahent.setEnabled(0)
                    self.btnRemoveContrahent.setEnabled(0)
                    self.btnUpdateContrahentInfo.setEnabled(0)
                    self.btnAdd3rdParty.setEnabled(0)
                    self.btnRemove3rdParty.setEnabled(0)
                    self.btnUpdate3rdPartyInfo.setEnabled(0)
        self.updateLists()


    def updateList(self, aList, aRole, selItem=''):
        if (selItem == ''):
            if aList.currentItem():
                selItem = aList.currentItem().text(0)
        aList.clear()
        hasSelected = 0
        quClients = kumula.DB.doSelect("cases_clients", ("clientid",), "clientid", {"caseid":"%i" % self.CaseID, "clientrole":"%i" % aRole})
        while quClients.next():
            myClientID   = quClients.value(0).toString()
            myClientName = ClientInterface.KClient(myClientID).clientName
            item = QListViewItem(aList)
            item.setText(0, myClientID)
            item.setText(1, myClientName)
            if (myClientID == selItem):
                aList.setSelected(item, 1)
                aList.setCurrentItem(item)
                hasSelected = 1
        if (hasSelected == 0):
            if aList.firstChild():
                aList.setCurrentItem(aList.firstChild())
                aList.setSelected(aList.firstChild(), 1)


    def updateLists(self):
        self.updateList(self.listClients,     0)
        self.updateList(self.listContrahents, 1)
        self.updateList(self.list3rdParty,    2)
        # manually load infos during initialisation
        self.loadInfo(self.listClients,     0, self.edClientInfo)
        self.loadInfo(self.listContrahents, 1, self.edContrahentInfo)
        self.loadInfo(self.list3rdParty,    2, self.ed3rdPartyInfo)


# global methods for adding and removing items

    def addItem(self, aList, aRole):
        dlg = KClientSelectDlg()
        if (dlg.exec_loop() == QDialog.Accepted):
            ClientID = dlg.listClients.selectedItem().text(0)
            kumula.DB.doInsert("cases_clients", {"caseid":"%i" % self.CaseID, "clientid":ClientID, "clientrole":"%i" % aRole})
            self.updateList(aList, aRole, ClientID)

    def removeItem(self, aList, aRole):
        if aList.currentItem():
            ClientID = aList.currentItem().text(0)
            myMsg = QString("Do you really want to remove<br><b>")
            myMsg.append(ClientInterface.KClient(ClientID).clientName)
            myMsg.append("</b> (")
            myMsg.append(ClientID)
            myMsg.append(")<br>from this case?")
            result = QMessageBox.question(self, "Removing...", myMsg, "&Remove", "&Cancel")
            if (result == 0):
                kumula.DB.doDelete("cases_clients", {"caseid":"%i" % self.CaseID, "clientid":ClientID, "clientrole":"%i" % aRole})
                self.updateList(aList, aRole)

    def loadInfo(self, aList, aRole, aTextEdit):
        if aList.currentItem():
            ClientID = aList.currentItem().text(0)
            quComments = kumula.DB.doSelect("cases_clients", ("comment",), "clientid", {"caseid":"%i" % self.CaseID, "clientid":ClientID, "clientrole":"%i" % aRole})
            while quComments.next():
                myComment = quComments.value(0).toString()
                aTextEdit.setText(myComment)
        else:
            aTextEdit.clear()


    def storeInfo(self, aList, aRole, aTextEdit):
        if aList.currentItem():
            ClientID = aList.currentItem().text(0)
            myComment = aTextEdit.text()
            kumula.DB.doUpdate("cases_clients", {"comment":myComment}, {"caseid":"%i" % self.CaseID, "clientid":ClientID, "clientrole":"%i" % aRole})


# CLIENTS

    def handleAddClient(self):
        self.addItem(self.listClients, 0)

    def handleRemoveClient(self):
        self.removeItem(self.listClients, 0)

    def handleClientChanged(self):
        self.loadInfo(self.listClients, 0, self.edClientInfo)

    def handleUpdateClientInfo(self):
        self.storeInfo(self.listClients, 0, self.edClientInfo)

# CONTRAHENTS

    def handleAddContrahent(self):
        self.addItem(self.listContrahents, 1)

    def handleRemoveContrahent(self):
        self.removeItem(self.listContrahents, 1)

    def handleContrahentChanged(self):
        self.loadInfo(self.listContrahents, 1, self.edContrahentInfo)

    def handleUpdateContrahentInfo(self):
        self.storeInfo(self.listContrahents, 1, self.edContrahentInfo)

# 3RD PARTIES

    def handleAdd3rdParty(self):
        self.addItem(self.list3rdParty, 2)

    def handleRemove3rdParty(self):
        self.removeItem(self.list3rdParty, 2)

    def handle3rdPartyChanged(self):
        self.loadInfo(self.list3rdParty, 2, self.ed3rdPartyInfo)

    def handleUpdate3rdPartyInfo(self):
        self.storeInfo(self.list3rdParty, 2, self.ed3rdPartyInfo)
