

from widgetTmpl_events import WidgetEvents
import kumula
from qt import *
from SpecialItems import KColoredListViewItem
import UserInterface
import Database

class CaseWidget(WidgetEvents):

    def __init__(self, parent):
        WidgetEvents.__init__(self, parent)
        self.setIcon(kumula.getSideBarIcon("events"))
        self.setCaption("Events")


    def setCaseID(self, aCaseID):
        self.CaseID = aCaseID
        if (self.CaseID > 0):
            # get responsible user for this case
            quCaseData = kumula.DB.doSelect("cases", ("userid","archived"), "", {"caseid":"%i" % self.CaseID})
            while quCaseData.next():
                myUserID   = quCaseData.value(0).toInt()
                myArchived = quCaseData.value(1).toInt()
                self.caseUser = UserInterface.getUserByID(myUserID).userName
                if (myArchived > 0):
                    self.btnAppNew.setEnabled(0)
                    self.btnAppUpdate.setEnabled(0)
                    self.btnAppDelete.setEnabled(0)
                    self.btnTimeLimitNew.setEnabled(0)
                    self.btnTimeLimitUpdate.setEnabled(0)
                    self.btnTimeLimitDelete.setEnabled(0)
                    self.btnResubNew.setEnabled(0)
                    self.btnResubUpdate.setEnabled(0)
                    self.btnResubDelete.setEnabled(0)
            # update all lists
            self.updateLists()


    def updateList(self, aList, aType, selItem=0):
        if (selItem == 0):
            if aList.currentItem():
                selItem = aList.currentItem().tag
        aList.clear()
        hasSelected = 0
        quEvents = kumula.DB.doSelect("cases_events", ("id","userid","date","time","length","comment","done"),
                                        "date", {"caseid":"%i" % self.CaseID, "eventtype":"%i" % aType})
        while quEvents.next():
            myID       = quEvents.value(0).toInt()
            myUserID   = quEvents.value(1).toInt()
            myDate     = quEvents.value(2).toDate()
            myTime     = quEvents.value(3).toTime()
            myLength   = quEvents.value(4).toInt()
            myComment  = quEvents.value(5).toString()
            myDone     = quEvents.value(6).toInt()
            myUserName = UserInterface.getUserByID(myUserID).getFormattedName()

            item = KColoredListViewItem(aList, myID)
            item.setText(0, myDate.toString("yyyy-MM-dd (ddd)"))
            if (aType == 0):
                item.setText(1, myTime.toString("hh:mm"))
                item.setText(2, myUserName)
            else:
                item.setText(1, myUserName)
            if (myDone == 0):
                item.setColor(QString("#000000"))
            else:
                item.setColor(QString("#808080"))

            if (myID == selItem):
                aList.setSelected(item, 1)
                aList.setCurrentItem(item)
                hasSelected = 1
        if (hasSelected == 0):
            if aList.firstChild():
                aList.setCurrentItem(aList.firstChild())
                aList.setSelected(aList.firstChild(), 1)


    def updateLists(self):
        # set "nice" values, if a list has no entries
        self.clearAppData()
        self.clearTimeLimitData()
        self.clearResubData()
        # update all lists
        self.updateList(self.listAppointments,  0)
        self.updateList(self.listTimeLimits,    1)
        self.updateList(self.listResubmissions, 2)
        # make sure the values are updated
        self.updateAppData()
        self.updateTimeLimitData()
        self.updateResubData()


    def initUserList(self, aCBuser, anUserName=''):
        preUserName = anUserName
        if (preUserName == ""):
            preUserName = kumula.UserName
        aCBuser.clear()
        allUsers = UserInterface.getAllUsers()
        for user in allUsers:
            myEntry = QString(user.getFormattedName())
            myEntry.append(' (')
            myEntry.append(user.userName)
            myEntry.append(')')
            aCBuser.insertItem(myEntry)
            if (user.userName == preUserName):
                aCBuser.setCurrentText(myEntry)

    def getSelectedUser(self, aCBuser):
        myEntry = aCBuser.currentText()
        startPos = myEntry.findRev('(')
        myUserName = myEntry.mid(startPos+1, myEntry.length()-startPos-2)
        return myUserName


# GLOBAL EVENT METHODS

    def newEvent(self, anEventType, anUserName, aComment, aDate, aTime='', aLength=0):
        kumula.DB.doInsert("cases_events", {"caseid":"%i" % self.CaseID,
                                            "eventtype":"%i" % anEventType,
                                            "userid":"%i" % UserInterface.KUser(anUserName).userID,
                                            "date":aDate,
                                            "time":aTime,
                                            "length":"%i" % aLength,
                                            "comment":aComment,
                                            "done":"0"})
        myResult = kumula.DB.lastAutoID()
        return myResult

    def updateEvent(self, anID, anUserName, aComment, aDate, aTime='', aLength=0):
        kumula.DB.doUpdate("cases_events", {"userid":"%i" % UserInterface.KUser(anUserName).userID,
                                            "date":aDate,
                                            "time":aTime,
                                            "length":"%i" % aLength,
                                            "comment":aComment}, {"id":"%i" % anID})

    def deleteEvent(self, evID):
        kumula.DB.doDelete("cases_events", {"id":"%i" % evID})


# APPOINTMENTS


    def clearAppData(self):
        # set "nice" values
        self.edAppDate.setDate(QDate.currentDate())
        self.edAppStart.setTime(QTime(12, 0, 0))
        self.edAppLength.setTime(QTime(0, 30, 0))
        self.edAppComment.clear()
        self.initUserList(self.cbAppUser, self.caseUser)

    def updateAppData(self):
        if self.listAppointments.currentItem():
            myAppID = self.listAppointments.currentItem().tag
            quEvent = kumula.DB.doSelect("cases_events", ("date","time","length","userid","comment"), "id", {"id":"%i" % myAppID})
            while quEvent.next():
                myDate    = quEvent.value(0).toDate()
                myStart   = quEvent.value(1).toTime()
                myLength  = quEvent.value(2).toInt()
                myUser    = quEvent.value(3).toInt()
                myComment = quEvent.value(4).toString()
                myHours = int(myLength / 60)
                myMinutes = myLength - (myHours * 60)

                self.edAppDate.setDate(QDate(myDate))
                self.edAppStart.setTime(QTime(myStart))
                self.edAppLength.setTime(QTime(myHours, myMinutes, 0))
                self.edAppComment.setText(myComment)
                self.initUserList(self.cbAppUser, UserInterface.getUserByID(myUser).userName)
        else:
            self.clearAppData()

    def handleAppNew(self):
        myLength = self.edAppLength.time()
        lenMinutes = myLength.hour()*60 + myLength.minute()
        newID = self.newEvent(0,
                              self.getSelectedUser(self.cbAppUser),
                              self.edAppComment.text(),
                              self.edAppDate.date().toString(Qt.ISODate),
                              self.edAppStart.time().toString(Qt.ISODate),
                              lenMinutes)
        self.updateList(self.listAppointments, 0, newID)

    def handleAppUpdate(self):
        if self.listAppointments.currentItem():
            myAppID = self.listAppointments.currentItem().tag
            myLength = self.edAppLength.time()
            lenMinutes = myLength.hour()*60 + myLength.minute()
            self.updateEvent(myAppID,
                             self.getSelectedUser(self.cbAppUser),
                             self.edAppComment.text(),
                             self.edAppDate.date().toString(Qt.ISODate),
                             self.edAppStart.time().toString(Qt.ISODate),
                             lenMinutes)
            self.updateList(self.listAppointments, 0, myAppID)

    def handleAppDelete(self):
        if self.listAppointments.currentItem():
            myAppID = self.listAppointments.currentItem().tag
            myMsg = QString("Do you really want to remove the selected appointment from this case?")
            result = QMessageBox.question(self, "Delete appointment", myMsg, "&Delete", "&Cancel")
            if (result == 0):
                self.deleteEvent(myAppID)
                self.updateList(self.listAppointments, 0)

# TIME LIMITS


    def clearTimeLimitData(self):
        self.edTimeLimitDate.setDate(QDate.currentDate())
        self.edTimeLimitComment.clear()
        self.initUserList(self.cbTimeLimitUser, self.caseUser)

    def updateTimeLimitData(self):
        if self.listTimeLimits.currentItem():
            myTimeLimitID = self.listTimeLimits.currentItem().tag
            quEvent = kumula.DB.doSelect("cases_events", ("date","userid","comment"), "id", {"id":"%i" % myTimeLimitID})
            while quEvent.next():
                myDate    = quEvent.value(0).toDate()
                myUser    = quEvent.value(1).toInt()
                myComment = quEvent.value(2).toString()

                self.edTimeLimitDate.setDate(QDate(myDate))
                self.edTimeLimitComment.setText(myComment)
                self.initUserList(self.cbTimeLimitUser, UserInterface.getUserByID(myUser).userName)
        else:
            self.clearTimeLimitData()

    def handleTimeLimitNew(self):
        newID = self.newEvent(1,
                              self.getSelectedUser(self.cbTimeLimitUser),
                              self.edTimeLimitComment.text(),
                              self.edTimeLimitDate.date().toString(Qt.ISODate))
        self.updateList(self.listTimeLimits, 1, newID)

    def handleTimeLimitUpdate(self):
        if self.listTimeLimits.currentItem():
            myTimeLimitID = self.listTimeLimits.currentItem().tag
            self.updateEvent(myTimeLimitID,
                             self.getSelectedUser(self.cbTimeLimitUser),
                             self.edTimeLimitComment.text(),
                             self.edTimeLimitDate.date().toString(Qt.ISODate))
            self.updateList(self.listTimeLimits, 1, myTimeLimitID)

    def handleTimeLimitDelete(self):
        if self.listTimeLimits.currentItem():
            myTimeLimitID = self.listTimeLimits.currentItem().tag
            myMsg = QString("Do you really want to remove the selected time limit from this case?")
            result = QMessageBox.question(self, "Delete time limit", myMsg, "&Delete", "&Cancel")
            if (result == 0):
                self.deleteEvent(myTimeLimitID)
                self.updateList(self.listTimeLimits, 1)



# RESUBMISSIONS


    def clearResubData(self):
        self.edResubDate.setDate(QDate.currentDate())
        self.edResubComment.clear()
        self.initUserList(self.cbResubUser, self.caseUser)

    def updateResubData(self):
        if self.listResubmissions.currentItem():
            myResubID = self.listResubmissions.currentItem().tag
            quEvent = kumula.DB.doSelect("cases_events", ("date","userid","comment"), "id", {"id":"%i" % myResubID})
            while quEvent.next():
                myDate    = quEvent.value(0).toDate()
                myUser    = quEvent.value(1).toInt()
                myComment = quEvent.value(2).toString()

                self.edResubDate.setDate(QDate(myDate))
                self.edResubComment.setText(myComment)
                self.initUserList(self.cbResubUser, UserInterface.getUserByID(myUser).userName)
        else:
            self.clearResubData()

    def handleResubNew(self):
        newID = self.newEvent(2,
                              self.getSelectedUser(self.cbResubUser),
                              self.edResubComment.text(),
                              self.edResubDate.date().toString(Qt.ISODate))
        self.updateList(self.listResubmissions, 2, newID)

    def handleResubUpdate(self):
        if self.listResubmissions.currentItem():
            myResubID = self.listResubmissions.currentItem().tag
            self.updateEvent(myResubID,
                             self.getSelectedUser(self.cbResubUser),
                             self.edResubComment.text(),
                             self.edResubDate.date().toString(Qt.ISODate))
            self.updateList(self.listResubmissions, 2, myResubID)

    def handleResubDelete(self):
        if self.listResubmissions.currentItem():
            myResubID = self.listResubmissions.currentItem().tag
            myMsg = QString("Do you really want to remove the selected resubmission date from this case?")
            result = QMessageBox.question(self, "Delete resubmission", myMsg, "&Delete", "&Cancel")
            if (result == 0):
                self.deleteEvent(myResubID)
                self.updateList(self.listResubmissions, 2)

