

from widgetTmpl_documents import WidgetDocuments
import kumula
from qt import *
from ClientInterface import KClient
from DocumentInterface import getFolderByRelation,createFolderRoot,KDocument,KDocVersion
from SpecialItems import KListViewItemWithID
from documents_InfoDialog import KDocumentInfoDialog


class CaseWidget(WidgetDocuments):

    def __init__(self, parent):
        WidgetDocuments.__init__(self, parent)
        self.setIcon(kumula.getSideBarIcon("documents"))
        self.lvDocList.setRootIsDecorated(1)


    def setCaseID(self, aCaseID):
        self.CaseID = aCaseID
        if (self.CaseID > 0):
            self.DocFolder = getFolderByRelation("cases", "%i" % self.CaseID)
            if not self.DocFolder:
                self.DocFolder = createFolderRoot("cases", "%i" % self.CaseID)
            self.updateList()
            self.initLetterMenu()

            quCaseData = kumula.DB.doSelect("cases", ("archived",), "caseid", {"caseid":"%i" % self.CaseID})
            while quCaseData.next():
                myArchived  = quCaseData.value(0).toInt()
                if (myArchived > 0):
                    self.btnNewLetter.setEnabled(0)
                    self.btnImportDocument.setEnabled(0)
                    self.btnImportVersion.setEnabled(0)
                    self.btnDelete.setEnabled(0)


    def updateList(self, anID=0, isVersionID=0):
        self.lvDocList.clear()
        if (self.DocFolder):
            myDocs = self.DocFolder.getDocuments()
            for aDoc in myDocs:
                item = KListViewItemWithID(self.lvDocList, aDoc.docID)
                item.setText(0, aDoc.title)
                item.setText(1, aDoc.logDate.toString(Qt.ISODate))
                item.setText(2, aDoc.comment)
                myVersions = aDoc.getVersions()
                myNumber = 1
                for aVersion in myVersions:
                    vitem = KListViewItemWithID(item, aVersion.versionID)
                    vitem.setText(0, "# %i" % myNumber)
                    vitem.setText(1, aVersion.time.date().toString(Qt.ISODate))
                    vitem.setText(2, aVersion.comment)
                    myNumber = myNumber+1


    def initLetterMenu(self):
        self.receiverMenu = QPopupMenu(self)
        curRec = 0
        oldRole = 0
        self.recList = {}
        quClients = kumula.DB.doSelect("cases_clients", ("clientid","clientrole"), "clientrole", {"caseid":"%i" % self.CaseID})
        while quClients.next():
            myClientID   = quClients.value(0).toString()
            myClientRole = quClients.value(1).toInt()
            myClientName = KClient(myClientID).clientName
            if (myClientRole > oldRole):
                oldRole = myClientRole
                self.receiverMenu.insertSeparator()
            myMenuStr = QString(myClientName)
            myMenuStr.append(" (")
            myMenuStr.append(myClientID)
            myMenuStr.append(")")
#            myMenuStr = "%s (%s)" % (myClientName, myClientID)
            recID = self.receiverMenu.insertItem(myMenuStr, self.handleNewLetter)
            self.receiverMenu.setItemParameter(recID, curRec)
            self.recList[curRec] = myClientID
            curRec = curRec+1
        self.btnNewLetter.setPopup(self.receiverMenu)


    def handleNewLetter(self, recID):
        kumula.runKumulaApp("letters", {"client":QString(self.recList[recID])})


    def importDocument(self):
        fileName = QFileDialog.getOpenFileName("~/", "All files (*.*)", kumula.MainWin, "OpenFileDlg", "Choose a file to import")
        if (fileName):
            dlg = KDocumentInfoDialog(self)
            dlg.setCaption("Import new document")
            dlg.edFilename.setText(fileName)
            dlg.edDate.setDate(QDate().currentDate())
            dlg.edDescription.clear()
            if dlg.exec_loop():
                newDoc = self.DocFolder.newDocument("", dlg.edFilename.text(), dlg.edDescription.text(), dlg.edDate.date(), "")
                if (newDoc):
                    newVersion = newDoc.importVersionFromFile(fileName, "First version")
                self.updateList()


    def importVersion(self):
        if (self.lvDocList.currentItem()):
            myDocItem = self.lvDocList.currentItem()
            if myDocItem.parent():
                # selected item is a version
                myDocItem = myDocItem.parent()
            myDoc = KDocument(myDocItem.tag)
            if (myDoc):
                fileName = QFileDialog.getOpenFileName("~/", "All files (*.*)", kumula.MainWin, "OpenFileDlg", "Choose a file to import")
                if (fileName):
                    dlg = KDocumentInfoDialog(self)
                    dlg.setCaption("Import document version")
                    dlg.edFilename.setText(myDoc.title)
                    dlg.edFilename.setEnabled(0)
                    dlg.edDate.setDate(myDoc.logDate)
                    dlg.edDate.setEnabled(0)
                    dlg.edDescription.clear()
                    dlg.lblDescription.setText("Comment for this version:")

                    if dlg.exec_loop():
                        newVersion = myDoc.importVersionFromFile(fileName, dlg.edDescription.text())
                    self.updateList()


    def exportDocVer(self):
        if (self.lvDocList.currentItem()):
            myDocItem = self.lvDocList.currentItem()
            if myDocItem.parent():
                # selected item is a version
                myVersion = KDocVersion(myDocItem.tag)
            else:
                # selected item is a document
                myVersion = KDocument(myDocItem.tag).getLatestVersion()
            myDocument = KDocument(myVersion.docID)
            myFileName = myDocument.title
            print "exporting %s (version %i)..." % (myFileName,myVersion.versionID)
            fileName = QFileDialog.getSaveFileName("~/", "All files (*.*)", self, "SaveFileDlg", "Input a filename to save")
            if (fileName):
                if (QFile.exists(fileName)):
                    result = QMessageBox.warning(self, "File exists already!", "Should the existing file be overwritten?", "&Overwrite", "&Cancel")
                    if (result > 0):
                        return
                myVersion.saveToFile(fileName)


    def deleteDocVer(self):
        if (self.lvDocList.currentItem()):
            myDocItem = self.lvDocList.currentItem()
            if myDocItem.parent():
                # selected item is a version
                myVersion = KDocVersion(myDocItem.tag)
                result = QMessageBox.warning(self, "Delete version", "Do you really want to delete this version?", "&Delete", "&Cancel")
                if (result == 0):
                    myVersion.delete()
                    self.updateList()
            else:
                # selected item is a document
                myDocument = KDocument(myDocItem.tag)
                result = QMessageBox.warning(self, "Delete document", "Do you really want to delete the document<br><b>%s</b><br>and all its versions?" % myDocument.title, "&Delete", "&Cancel")
                if (result == 0):
                    myDocument.delete()
                    self.updateList()

