

from widgetTmpl_journal import WidgetJournal
import kumula
from qt import *
from SpecialItems import KListViewItemWithID


class CaseWidget(WidgetJournal):

    def __init__(self, parent):
        WidgetJournal.__init__(self, parent)
        self.setIcon(kumula.getSideBarIcon("journal"))
        self.listRecords.setColumnAlignment(3, Qt.AlignRight)
        self.listRecords.setColumnAlignment(4, Qt.AlignRight)


    def setCaseID(self, aCaseID):
        self.CaseID = aCaseID
        if (self.CaseID > 0):
            quCaseData = kumula.DB.doSelect("cases", ("archived",), "caseid", {"caseid":"%i" % self.CaseID})
            while quCaseData.next():
                myArchived  = quCaseData.value(0).toInt()
                if (myArchived > 0):
                    pass
                    self.btnInsert.setEnabled(0)
                    self.btnUpdate.setEnabled(0)
                    self.btnDelete.setEnabled(0)
            self.updateRecordList()



    def updateRecordList(self, selItem=0):
        # first, make a dictionary of record types for displaying in the list
        myRecordTypes = {}
        quRecordTypes = kumula.DB.doSelect("cases_recordtypes", ("id","title"), "id")
        while quRecordTypes.next():
            myID    = quRecordTypes.value(0).toInt()
            myTitle = quRecordTypes.value(1).toString()
            myRecordTypes[myID] = myTitle

        # second, update the list
        if (selItem == 0):
            if self.listRecords.currentItem():
                selItem = self.listRecords.currentItem().tag
        self.listRecords.clear()
        self.clearRecordData()
        mySaldo = 0
        hasSelected = 0
        quRecords = kumula.DB.doSelect("cases_records", ("id","date","value","credeb","comment","recordtype"),
                                        "date", {"caseid":"%i" % self.CaseID})
        while quRecords.next():
            myID         = quRecords.value(0).toInt()
            myDate       = quRecords.value(1).toDate()
            myValue      = quRecords.value(2).toInt()
            myCreDeb     = quRecords.value(3).toInt()
            myComment    = quRecords.value(4).toString()
            myRecordType = quRecords.value(5).toInt()

            item = KListViewItemWithID(self.listRecords, myID)
            item.setText(0, myDate.toString(Qt.ISODate))
            item.setText(1, myComment)
            item.setText(2, myRecordTypes[myRecordType])
            myCommified = self.CommifyValue(myValue)
            if (myCreDeb > 0):
                item.setText(3, myCommified)
                item.setText(4, " ")
                mySaldo = mySaldo + myValue
            else:
                item.setText(3, " ")
                item.setText(4, myCommified)
                mySaldo = mySaldo - myValue

            if (myID == selItem):
                self.listRecords.setSelected(item, 1)
                self.listRecords.setCurrentItem(item)
                hasSelected = 1
        if (hasSelected == 0):
            if self.listRecords.firstChild():
                self.listRecords.setCurrentItem(self.listRecords.firstChild())
                self.listRecords.setSelected(self.listRecords.firstChild(), 1)
        self.loadRecordData()
        self.lblSaldo.setText(self.CommifyValue(mySaldo))


    def loadRecordData(self):
        if self.listRecords.currentItem():
            myRecID = self.listRecords.currentItem().tag
            quRecord = kumula.DB.doSelect("cases_records", ("date","comment","recordtype","value","credeb"), "id", {"id":"%i" % myRecID})
            while quRecord.next():
                myDate    = quRecord.value(0).toDate()
                myComment = quRecord.value(1).toString()
                myType    = quRecord.value(2).toInt()
                myValue   = quRecord.value(3).toInt()
                myCreDeb  = quRecord.value(4).toInt()
                self.edDate.setDate(myDate)
                self.edDescription.setText(myComment)
                self.edValue.setValue(myValue)
                self.cbCreDeb.setCurrentItem(myCreDeb)
                self.initTypeList(myType)
        else:
            self.clearRecordData()

    def clearRecordData(self):
        self.edDate.setDate(QDate.currentDate())
        self.edDescription.clear()
        self.edValue.setValue(0)
        self.cbCreDeb.setCurrentItem(0)
        self.initTypeList()


    def NOCommifyValue(self, aValue):
        if (aValue < 0):
            valueStr = QString("%i" % (aValue*-1))
        else:
            valueStr = QString("%i" % aValue)
        i = valueStr.length()-2
        valueStr.insert(i, ",")
        while (i > 3):
            i = i-3
            valueStr.insert(i, ".")
        if (aValue < 0):
            valueStr.prepend("- ")
        return valueStr


    def CommifyValue(self, aValue):
        return QString("%.2f" % (aValue/100.0))



    def initTypeList(self, aType=0):
        self.cbType.clear()
        quTypes = kumula.DB.doSelect("cases_recordtypes", ("id","title"), "title")
        while quTypes.next():
            myID    = quTypes.value(0).toInt()
            myTitle = quTypes.value(1).toString()
            myEntry = QString(myTitle)
            myEntry.append(' (')
            myEntry.append("%i" % myID)
            myEntry.append(')')
            self.cbType.insertItem(myEntry)
            if (myID == aType):
                self.cbType.setCurrentText(myEntry)

    def getSelectedType(self):
        myEntry = self.cbType.currentText()
        startPos = myEntry.findRev('(')
        myType = myEntry.mid(startPos+1, myEntry.length()-startPos-2).toInt()[0]
        return myType



    def insertRecord(self):
        kumula.DB.doInsert("cases_records", {"caseid":"%i" % self.CaseID,
                                             "date":self.edDate.date().toString(Qt.ISODate),
                                             "value":"%i" % self.edValue.value(),
                                             "credeb":"%i" % self.cbCreDeb.currentItem(),
                                             "comment":self.edDescription.text(),
                                             "recordtype":"%i" % self.getSelectedType()})
        myID = kumula.DB.lastAutoID()
        self.updateRecordList(myID)


    def updateRecord(self):
        myItem = self.listRecords.currentItem()
        if (myItem):
            myID = myItem.tag
            kumula.DB.doUpdate("cases_records", {"date":self.edDate.date().toString(Qt.ISODate),
                                                 "value":"%i" % self.edValue.value(),
                                                 "credeb":"%i" % self.cbCreDeb.currentItem(),
                                                 "comment":self.edDescription.text(),
                                                 "recordtype":"%i" % self.getSelectedType()}, {"id":"%i" % myID})
            self.updateRecordList(myID)


    def deleteRecord(self):
        myItem = self.listRecords.currentItem()
        if (myItem):
            myID = myItem.tag
            myMsg = QString("Do you really want to remove the selected record from the journal?")
            result = QMessageBox.question(self, "Delete record", myMsg, "&Delete", "&Cancel")
            if (result == 0):
                kumula.DB.doDelete("cases_records", {"id":"%i" % myID})
                self.updateRecordList()


