
from CaseTypeDialogTmpl import KCaseTypeDialogTmpl
from SpecialItems import KListViewItemWithID
import kumula
from qt import QMessageBox


class KCaseTypeDialog(KCaseTypeDialogTmpl):

    def __init__(self, parent):
        KCaseTypeDialogTmpl.__init__(self, parent)
        self.updateCaseTypes()


    def updateCaseTypes(self, selectedType=0):
        if (selectedType == 0):
            if self.lvCaseTypes.currentItem():
                selectedType = self.lvCaseTypes.currentItem().tag
        self.lvCaseTypes.clear()
        quCaseTypes = kumula.DB.doSelect("cases_casetypes", ("id","title"), "title")
        while quCaseTypes.next():
            myID    = quCaseTypes.value(0).toInt()
            myTitle = quCaseTypes.value(1).toString()
            item = KListViewItemWithID(self.lvCaseTypes, myID)
            item.setText(0, myTitle)
            if (myID == selectedType):
                self.lvCaseTypes.setSelected(item, 1)
                self.lvCaseTypes.setCurrentItem(item)



    def updateCaseTypeTitle(self):
        curItem = self.lvCaseTypes.currentItem()
        if curItem:
            self.edCaseTypeTitle.setText(curItem.text(0))


    def handleAdd(self):
        myTitle = self.edCaseTypeTitle.text().stripWhiteSpace()
        if (myTitle.length() > 0):
            kumula.DB.doInsert("cases_casetypes", {"title":myTitle})
            self.updateCaseTypes(kumula.DB.lastAutoID())
            self.edCaseTypeTitle.clear()
        else:
            QMessageBox.critical(kumula.MainWin, "Error", "No valid title<br>for a case type.")
        self.edCaseTypeTitle.setFocus()


    def handleRename(self):
        curItem = self.lvCaseTypes.currentItem()
        if curItem:
            myNewTitle = self.edCaseTypeTitle.text().stripWhiteSpace()
            if (myNewTitle.length() > 0):
                kumula.DB.doUpdate("cases_casetypes", {"title":myNewTitle}, {"id":"%i" % curItem.tag}, 1)
                self.updateCaseTypes(curItem.tag)
                self.edCaseTypeTitle.clear()
            else:
                QMessageBox.critical(kumula.MainWin, "Error", "No valid title<br>for a case type.")
        self.edCaseTypeTitle.setFocus()


    def handleDelete(self):
        curItem = self.lvCaseTypes.currentItem()
        if curItem:
            casesForThisType = kumula.DB.doCount("cases", {"type":"%i" % curItem.tag})
            if (casesForThisType > 0):
                QMessageBox.critical(kumula.MainWin, "Error", "There are <b>%i</b> cases with this type.<br><br>Please mark them as another type,<br>after that you can delete this type." % casesForThisType)
            else:
                kumula.DB.doDelete("cases_casetypes", {"id":"%i" % curItem.tag}, 1)
                self.updateCaseTypes()
        self.lvCaseTypes.setFocus()


