# ============================================================
# ListAppointments.py
# ------------------------------------------------------------
# Copyright (C) 2005 Juergen Gerner.
# Licensed under the LGPL.
# ============================================================

import kumula
from qt import *
import UserInterface

# ------------------------------------------------------------


tmplMissedApp = '<b>DATE</b> (TIMEFROM):<br><a href="case.CASEID">TITLE</a><br>COMMENT'
tmplTodayApp = '<b>TIMEFROM - TIMETO:</b><br><a href="case.CASEID">TITLE</a><br>COMMENT'
tmplTomorrowApp = tmplTodayApp


class KListAppointments(QTextBrowser):


    def formatEntry(self, tmplStr, anID, aCaseID, aUserID, aDate, aTime, aLength, aComment, aDone, aHighlightFlag):
        quCase = kumula.DB.doSelect("cases", ("title",), "title", {"caseid":"%i" % aCaseID})
        while quCase.next():
            myTitle = quCase.value(0).toString()
        myUser = UserInterface.getUserByID(aUserID)
        myToTime = aTime.addSecs(aLength*60)
        myContent = QString(tmplStr)
        myContent.replace("DATE", aDate.toString())
        myContent.replace("TIMEFROM", aTime.toString("hh:mm"))
        myContent.replace("TIMETO", myToTime.toString("hh:mm"))
        myContent.replace("TITLE", myTitle)
        myContent.replace("USERNAME", myUser.getFormattedName())
        myContent.replace("CASEID", "%i" % aCaseID)
        myContent.replace("COMMENT", aComment)
        if (aHighlightFlag % 2) > 0:
            color = QString('#DDDDDD')
        else:
            color = QString('#FFFFFF')
        content = QString('<tr><td bgcolor=')
        content.append(color)
        content.append('>')
        content.append(myContent)
        if (self.showAllUsers > 0):
            content.append('<br><b>User: </b>')
            content.append(myUser.getFormattedName())
        content.append('</td></tr>')
        if (aDate <= QDate.currentDate()) and (aDone == 0):
            content.append('<tr><td align="right" bgcolor=')
            content.append(color)
            content.append('><a href="done.')
            content.append('%i' % anID)
            content.append('">mark as done</a></td></tr>')
        return content


    def formatDay(self, aTitle, aContent):
        content = QString("<td width=33%><table border=0 cellpadding=5 cellspacing=0><tr><td><b>")
        content.append(aTitle)
        content.append("</b></td></tr>")
        content.append(aContent)
        content.append("</table></td>")
        return content


    def updateContent(self):
        self.showAllUsers = kumula.ProgramSettings.readInt("view", "showAllEvents",  0)

        content = QString('<html><body bgcolor="#FFFFFF" text="#000000" link="#0000FF" alink="#FF0000" vlink="#0000FF"><font size=+1>Appointments for ')
        if (self.showAllUsers == 0):
            myUser = UserInterface.KUser(kumula.UserName)
            content.append(myUser.getFormattedName())
        else:
            content.append('all users')
        content.append('</font><hr><table border=0 cellpadding=3 cellspacing=1 width=100%><tr>')

        countMissed = 1
        countToday = 1
        countTomorrow = 1
        myMissedEntries = QString("")
        myTodayEntries = QString("")
        myTomorrowEntries = QString("")
        if (self.showAllUsers == 0):
            myCause = {"eventtype":"0", "userid":"%i" % myUser.userID}
        else:
            myCause = {"eventtype":"0"}
        quAppointments = kumula.DB.doSelect("cases_events", ("id","caseid","userid","date","time","length","comment","done"), "date", myCause)
        while quAppointments.next():
            myID      = quAppointments.value(0).toInt()
            myCaseID  = quAppointments.value(1).toInt()
            myUserID  = quAppointments.value(2).toInt()
            myDate    = quAppointments.value(3).toDate()
            myTime    = quAppointments.value(4).toTime()
            myLength  = quAppointments.value(5).toInt()
            myComment = quAppointments.value(6).toString()
            myDone    = quAppointments.value(7).toInt()
            if (myDate < QDate.currentDate()) and (myDone == 0):
                myMissedEntries.append(self.formatEntry(tmplMissedApp, myID, myCaseID, myUserID, myDate, myTime, myLength, myComment, myDone, countMissed))
                countMissed = countMissed + 1
            if (myDate == QDate.currentDate()):
                myTodayEntries.append(self.formatEntry(tmplTodayApp, myID, myCaseID, myUserID, myDate, myTime, myLength, myComment, myDone, countToday))
                countToday = countToday + 1
            if (myDate == QDate.currentDate().addDays(1)):
                myTomorrowEntries.append(self.formatEntry(tmplTomorrowApp, myID, myCaseID, myUserID, myDate, myTime, myLength, myComment, myDone, countTomorrow))
                countTomorrow = countTomorrow + 1

        content.append(self.formatDay("Missed appointments:", myMissedEntries))
        content.append(self.formatDay("Appointments for today:", myTodayEntries))
        content.append(self.formatDay("Appointments for tomorrow:", myTomorrowEntries))

#        content.append(QString('<td>TEST</td>'))
        content.append('</tr></table></body></html>')
        self.setText(content)


    
    def handleAction(self, command, id):
        print "Command", command, "with ID", id
        if (command == "case"):
            kumula.runKumulaApp("casedata", {"case":id})
        if (command == "done"):
            result = QMessageBox.information(self, "Mark as done", "This appointment will be marked as done. Done appointments in the past will disappear from this list.", "Mark as &done", "&Cancel")
            if (result == 0):
                kumula.DB.doUpdate("cases_events", {"done":"1"}, {"id":id})
                self.updateContent()




    #############################################################################
    # for internal use!
    #############################################################################


    def createURL(self, context, action, id):   # 3x String
        url = QString(action)
        url.append(".")
        url.append(context)
        url.append(".")
        url.append(id)
        return url


    def setSource(self, url):
        params = QStringList.split('.', url)
        self.handleAction(params[0], params[1])


    def unknownCommand(self):
        QMessageBox.critical(kumula.MainWin, "Error", "Unknown command!")



    def reloadContent(self):
        self.loadContent(self.cID)


    def clearContent(self):
        self.setText('')

