
# ============================================================
# ListTimeLimits.py
# ------------------------------------------------------------
# Copyright (C) 2005 Juergen Gerner.
# Licensed under the LGPL.
# ============================================================

import kumula
from qt import *
import UserInterface

# ------------------------------------------------------------


tmplMissedEvent = '<b><font color=#A00000>DATE:</font></b><br><a href="case.CASEID">TITLE</a><br><font color=#800000>COMMENT</font>'
tmplTodayEvent  = '<a href="case.CASEID">TITLE</a><br>COMMENT'
tmplFutureEvent = '<b>DATE:</b><br><a href="case.CASEID">TITLE</a><br>COMMENT'


class KListTimeLimits(QTextBrowser):


    def formatTitle(self, aTitle):
        content = QString('<tr><td bgcolor=#AAAAAA><font color=#FFFFFF><b>')
        content.append(aTitle)
        content.append('</b></font></td></tr>')
        return content


    def formatEntry(self, tmplStr, anID, aCaseID, aUserID, aDate, aComment):
        quCase = kumula.DB.doSelect("cases", ("title",), "title", {"caseid":"%i" % aCaseID})
        while quCase.next():
            myTitle = quCase.value(0).toString()
        myUser = UserInterface.getUserByID(aUserID)

        myContent = QString(tmplStr)
        myContent.replace("DATE", aDate.toString())
        myContent.replace("TITLE", myTitle)
        myContent.replace("USERNAME", myUser.getFormattedName())
        myContent.replace("CASEID", "%i" % aCaseID)
        myContent.replace("COMMENT", aComment)
        content = QString('<tr><td>')
        content.append(myContent)
        if (self.showAllUsers > 0):
            content.append('<br><b>User: </b>')
            content.append(myUser.getFormattedName())
        content.append('<br><a href="done.')
        content.append('%i' % anID)
        content.append('">mark as done</a>')
        content.append('</td></tr>')
        return content



    def updateContent(self):
        self.showAllUsers = kumula.ProgramSettings.readInt("view", "showAllEvents",  0)
        myUser = UserInterface.KUser(kumula.UserName)

        content = QString('''<html>
            <body bgcolor="#FFFFFF" text="#000000" link="#0000FF" alink="#FF0000" vlink="#0000FF">
            <table border=0 cellpadding=3 cellspacing=1 width=100%><tr>
            ''')

# TIME LIMITS

        content.append('<td width=50%><font size=+1>Time limits related to ')
        if (self.showAllUsers == 0):
            content.append(myUser.getFormattedName())
        else:
            content.append('all users')
        content.append('</font><hr><table border=0 cellpadding=5 cellspacing=0>')

        currentState = -1
        # 0 = missed, 1 = today, 2 = future
        if (self.showAllUsers == 0):
            myCause = {"eventtype":"1", "done":"0", "userid":"%i" % myUser.userID}
        else:
            myCause = {"eventtype":"1", "done":"0"}
        quTimeLimits = kumula.DB.doSelect("cases_events", ("id","caseid","userid","date","comment"), "date", myCause)
        while quTimeLimits.next():
            myID      = quTimeLimits.value(0).toInt()
            myCaseID  = quTimeLimits.value(1).toInt()
            myUserID  = quTimeLimits.value(2).toInt()
            myDate    = quTimeLimits.value(3).toDate()
            myComment = quTimeLimits.value(4).toString()
            if (myDate < QDate.currentDate()):
                if (currentState < 0):
                    currentState = 0
                    content.append(self.formatTitle("Missed time limits:"))
                content.append(self.formatEntry(tmplMissedEvent, myID, myCaseID, myUserID, myDate, myComment))
            if (myDate == QDate.currentDate()):
                if (currentState < 1):
                    currentState = 1
                    content.append(self.formatTitle("Today:"))
                content.append(self.formatEntry(tmplTodayEvent, myID, myCaseID, myUserID, myDate, myComment))
            if (myDate > QDate.currentDate()):
                if (currentState < 2):
                    currentState = 2
                    content.append(self.formatTitle("Future time limits:"))
                content.append(self.formatEntry(tmplFutureEvent, myID, myCaseID, myUserID, myDate, myComment))
        content.append('</table></td>')

# RESUBMISSIONS

        content.append('<td width=50%><font size=+1>Resubmissions related to ')
        if (self.showAllUsers == 0):
            content.append(myUser.getFormattedName())
        else:
            content.append('all users')
        content.append('</font><hr><table border=0 cellpadding=5 cellspacing=0>')

        currentState = -1
        # 0 = missed, 1 = today, 2 = future
        if (self.showAllUsers == 0):
            myCause = {"eventtype":"2", "done":"0", "userid":"%i" % myUser.userID}
        else:
            myCause = {"eventtype":"2", "done":"0"}
        quResubs = kumula.DB.doSelect("cases_events", ("id","caseid","userid","date","comment"), "date", myCause)
        while quResubs.next():
            myID      = quResubs.value(0).toInt()
            myCaseID  = quResubs.value(1).toInt()
            myUserID  = quResubs.value(2).toInt()
            myDate    = quResubs.value(3).toDate()
            myComment = quResubs.value(4).toString()
            if (myDate < QDate.currentDate()):
                if (currentState < 0):
                    currentState = 0
                    content.append(self.formatTitle("Missed resubmissions:"))
                content.append(self.formatEntry(tmplMissedEvent, myID, myCaseID, myUserID, myDate, myComment))
            if (myDate == QDate.currentDate()):
                if (currentState < 1):
                    currentState = 1
                    content.append(self.formatTitle("Today:"))
                content.append(self.formatEntry(tmplTodayEvent, myID, myCaseID, myUserID, myDate, myComment))
            if (myDate > QDate.currentDate()):
                if (currentState < 2):
                    currentState = 2
                    content.append(self.formatTitle("Future resubmissions:"))
                content.append(self.formatEntry(tmplFutureEvent, myID, myCaseID, myUserID, myDate, myComment))
        content.append('</table></td>')

# -------------

        content.append('</tr></table></body></html>')
        self.setText(content)


    
    def handleAction(self, command, id):
        print "Command", command, "with ID", id
        if (command == "case"):
            kumula.runKumulaApp("casedata", {"case":id})
        if (command == "done"):
            result = QMessageBox.information(self, "Mark as done", "This event will be marked as done. Done events will disappear from this list.", "Mark as &done", "&Cancel")
            if (result == 0):
                kumula.DB.doUpdate("cases_events", {"done":"1"}, {"id":id})
                self.updateContent()




    #############################################################################
    # for internal use!
    #############################################################################


    def createURL(self, context, action, id):   # 3x String
        url = QString(action)
        url.append(".")
        url.append(context)
        url.append(".")
        url.append(id)
        return url


    def setSource(self, url):
        params = QStringList.split('.', url)
        self.handleAction(params[0], params[1])


    def unknownCommand(self):
        QMessageBox.critical(kumula.MainWin, "Error", "Unknown command!")



    def reloadContent(self):
        self.loadContent(self.cID)


    def clearContent(self):
        self.setText('')

