
# ============================================================
# MainWindow.py
# main window for the case management program.
# ------------------------------------------------------------
# Copyright (C) 2005-2006 Juergen Gerner.
# ============================================================

import kumula
from qt import *
from SideBar import KSideBar
from SpecialItems import KListViewItemWithID
from KumulaMainWindow import KKumulaMainWindow,KAction
from ListAppointments import KListAppointments
from ListTimeLimits import KListTimeLimits
from CaseSettingsDialog import KCaseSettingsDialog
from CaseTypeDialog import KCaseTypeDialog
from RecordTypeDialog import KRecordTypeDialog
from UserInterface import getUserByID,KUser

# ------------------------------------------------------------


class KMainWindow(KKumulaMainWindow):

    def getProgramName(self):
        return "Cases"

    def getProgramVersion(self):
        return "0.1"

    def getProgramCopyright(self):
        return "Copyright (C) 2005-2006 Juergen Gerner.<br>This program is part of my diploma thesis at the<br>FH Nuremberg, University of Applied Sciences.<br>Supervisor: Prof. Dr. Walter Loesel"


    def initSettings(self):
        KKumulaMainWindow.initSettings(self)
        self.lastCase      = kumula.ProgramSettings.readInt("view", "lastCase",      0)
        self.lastView      = kumula.ProgramSettings.readInt("view", "lastView",      0)
        self.showAllCases  = kumula.ProgramSettings.readInt("view", "showAllCases",  1)
        self.showAllEvents = kumula.ProgramSettings.readInt("view", "showAllEvents", 0)


    def createUI(self):
        KKumulaMainWindow.createUI(self)
        self.statusBar()
        # initalize user interface

        self.setCentralWidget(QWidget(self, "qt_central_widget"))
        myLayout = QHBoxLayout(self.centralWidget())

        self.folderView = KSideBar(self.centralWidget(), "folderView")
        self.folderView.insertButtonAuto("Active cases", "cases_active", 0)
        self.folderView.insertButtonAuto("Closed cases", "cases_closed", 1)
        self.folderView.insertSpacer()
        self.folderView.insertButtonAuto("Appointments", "appointments", 8)
        self.folderView.insertButtonAuto("Time limits", "timelimits", 9)
        myLayout.addWidget(self.folderView)

        self.widgetStack = QWidgetStack(self.centralWidget())

        self.caseList = QListView(self.widgetStack)
        self.caseList.addColumn("ID")
        self.caseList.addColumn("Title")
        self.caseList.addColumn("Lawyer")
        self.caseList.addColumn("Type")
        self.caseList.addColumn("Creation date")
        self.caseList.setAllColumnsShowFocus(1)
        self.caseList.setResizeMode(QListView.AllColumns)
        self.widgetStack.addWidget(self.caseList)

        self.caseArchieve = QListView(self.widgetStack)
        self.caseArchieve.addColumn("ID")
        self.caseArchieve.addColumn("Title")
        self.caseArchieve.addColumn("Lawyer")
        self.caseArchieve.addColumn("Type")
        self.caseArchieve.addColumn("Creation date")
        self.caseArchieve.setAllColumnsShowFocus(1)
        self.caseArchieve.setResizeMode(QListView.AllColumns)
        self.widgetStack.addWidget(self.caseArchieve)

        self.listAppointments = KListAppointments(self.widgetStack)
        self.widgetStack.addWidget(self.listAppointments)

        self.listTimeLimits = KListTimeLimits(self.widgetStack)
        self.widgetStack.addWidget(self.listTimeLimits)

        myLayout.addWidget(self.widgetStack)

        self.connect(self.folderView, PYSIGNAL("selectionChanged()"), self.updateView)
        self.connect(self.caseList, SIGNAL("doubleClicked()"), self.slotCaseListDoubleClick)
        self.folderView.selectButton(self.lastView)

        self.resize(QSize(600,482).expandedTo(self.minimumSizeHint()))
        self.clearWState(Qt.WState_Polished)



    def createMenuBar(self):
        KKumulaMainWindow.createMenuBar(self)

        # initialize Actions
        self.actCaseCreate            = KAction(self, "case_create",          "&New...",                    self.handleCaseNew,             "Create a new case.")
        self.actCaseEdit              = KAction(self, "case_edit",            "&Open...",                   self.handleCaseOpen,            "Open the selected case.")
        self.actCaseActivate          = KAction(self, "case_activate",        "&Activate again",            self.handleCaseActivate,        "Re-activates a closed case.")
        self.actCaseClose             = KAction(self, "case_close",           "&Close",                     self.handleCaseClose,           "Close this case.")
        self.actCaseDelete            = KAction(self, "case_delete",          "&Delete this case...",       None,                           "Delete the selected case.")
        self.actViewCasesAllUsers     = KAction(self, "view_casesallusers",   "Show &cases of all users",   self.handleViewCasesAllUsers,   "Toggles the view between your cases and the cases of all users.")
        self.actViewEventsAllUsers    = KAction(self, "view_eventsallusers",  "Show &events of all users",  self.handleViewEventsAllUsers,  "Toggles the view between your events and the cases of all users.")
        self.actViewRefresh           = KAction(self, "refresh",              "&Refresh",                   self.handleViewRefresh,         "Refresh all views.")
        self.actSettingsCaseTypes     = KAction(self, "settings_casetypes",   "Case &types...",             self.handleSettingsCaseTypes,   "Manage the various case types.")
        self.actSettingsRecordTypes   = KAction(self, "settings_recordtypes", "&Record types...",           self.handleSettingsRecordTypes, "Manage the various record types.")
        self.actSettingsAutoID        = KAction(self, "settings_autoid",      "&Auto IDs...",               self.handleSettingsAutoID,      "Configure the auto-id generator.")
        self.actExtrasRunClients      = KAction(self, "run_clients",          "&Client management",         self.handleExtrasRunClients,    "Run Kumula's client management program.")
        self.actExtrasRunInstitutions = KAction(self, "run_institutions",     "&Institution management",    None,                           "Run Kumula's institution management program.")
        self.actExtrasRunUsers        = KAction(self, "run_users",            "&User management",           self.handleExtrasRunUsers,      "Run Kumula's user management program.")
        self.actExtrasRunCalc         = KAction(self, "run_calc",             "Business &calculator",       self.handleExtrasRunCalc,       "Run Kumula's business calculator.")

        self.actViewCasesAllUsers.setToggleAction(1)
        self.actViewEventsAllUsers.setToggleAction(1)
        self.updateToggleButtons()

        # create Menu Structure
        self.menuCase = self.makePopupMenu()
        self.actCaseCreate.addToMenuBar(self.menuCase)
        self.actCaseEdit.addToMenuBar(self.menuCase)
        self.menuCase.insertSeparator()
        self.actCaseClose.addToMenuBar(self.menuCase)
        self.actCaseActivate.addToMenuBar(self.menuCase)
#        self.menuCase.insertSeparator()
#        self.actCaseDelete.addToMenuBar(self.menuCase)
        self.MenuBar.insertItem(QString("&Case"), self.menuCase, 0)

        self.menuView = self.makePopupMenu()
        self.actViewCasesAllUsers.addToMenuBar(self.menuView)
        self.actViewEventsAllUsers.addToMenuBar(self.menuView)
        self.menuView.insertSeparator()
        self.actViewRefresh.addToMenuBar(self.menuView)
        self.MenuBar.insertItem(QString("&View"), self.menuView, 1)

        self.menuSettings = self.makePopupMenu()
        self.actSettingsCaseTypes.addToMenuBar(self.menuSettings)
        self.actSettingsRecordTypes.addToMenuBar(self.menuSettings)
#        self.actSettingsAutoID.addToMenuBar(self.menuSettings)
        self.MenuBar.insertItem(QString("&Settings"), self.menuSettings, 2)

        self.menuExtras = self.makePopupMenu()
        self.actExtrasRunClients.addToMenuBar(self.menuExtras)
#        self.actExtrasRunInstitutions.addToMenuBar(self.menuExtras)
        self.actExtrasRunUsers.addToMenuBar(self.menuExtras)
        self.actExtrasRunCalc.addToMenuBar(self.menuExtras)
        self.MenuBar.insertItem(QString("E&xtras"), self.menuExtras, 3)

        # Create Popup Menu for docList
        self.menuCaseList = QPopupMenu(self)
        self.actCaseEdit.addToMenuBar(self.menuCaseList)



    def createToolBar(self):
        KKumulaMainWindow.createToolBar(self)
        self.actCaseCreate.addToToolBar(self.ToolBar, "New case")
        self.actCaseEdit.addToToolBar(self.ToolBar, "Open case")
        self.ToolBar.addSeparator()
        self.actCaseClose.addToToolBar(self.ToolBar, "Close case")
        self.actCaseActivate.addToToolBar(self.ToolBar, "Re-activate")
        self.ToolBar.addSeparator()
        self.actViewCasesAllUsers.addToToolBar(self.ToolBar, "All cases")
        self.actViewEventsAllUsers.addToToolBar(self.ToolBar, "All events")
        self.ToolBar.addSeparator()
        self.actViewRefresh.addToToolBar(self.ToolBar, "Refresh")



    def __tr(self,s,c = None):
        return qApp.translate("MainWindow",s,c)


    def updateView(self):
        self.lastView = self.folderView.currentButton()
        kumula.ProgramSettings.writeInt("view", "lastView", self.lastView)
        if (self.lastView == 0):
            self.updateCaseList()
            self.widgetStack.raiseWidget(self.caseList)
            self.actCaseEdit.setEnabled(1)
            self.actCaseActivate.setEnabled(0)
            self.actCaseClose.setEnabled(1)
        elif (self.lastView == 1):
            self.updateCaseList()
            self.widgetStack.raiseWidget(self.caseArchieve)
            self.actCaseEdit.setEnabled(1)
            self.actCaseActivate.setEnabled(1)
            self.actCaseClose.setEnabled(0)
        elif (self.lastView == 8):
            self.actCaseEdit.setEnabled(0)
            self.actCaseActivate.setEnabled(0)
            self.actCaseClose.setEnabled(0)
            self.listAppointments.updateContent()
            self.widgetStack.raiseWidget(self.listAppointments)
        elif (self.lastView == 9):
            self.actCaseEdit.setEnabled(0)
            self.actCaseActivate.setEnabled(0)
            self.actCaseClose.setEnabled(0)
            self.listTimeLimits.updateContent()
            self.widgetStack.raiseWidget(self.listTimeLimits)
        else:
            print "ERROR: Unknown view!"


    def updateCaseTypes(self):
        self.caseTypes = {}
        quCaseTypes = kumula.DB.doSelect("cases_casetypes", ("id","title"), "id")
        while quCaseTypes.next():
            myID    = quCaseTypes.value(0).toInt()
            myTitle = quCaseTypes.value(1).toString()
            self.caseTypes[myID] = myTitle


    def updateCaseList(self, selectedCase=0):
        self.updateCaseTypes()
        myCaseTypes = self.folderView.currentButton()
        if (myCaseTypes == 0):
            myCaseList = self.caseList
        else:
            myCaseList = self.caseArchieve

        # save ID of the selected case, and if possible, select it after updating the list
        if (selectedCase == 0):
            if myCaseList.currentItem():
                selectedCase = myCaseList.currentItem().tag
            else:
                # this is when the program starts
                selectedCase = self.lastCase
        myCaseList.clear()
        hasSelected = 0

        if (self.showAllCases == 0):
            myUser = KUser(kumula.UserName)
            myCause = {"archived":"%i" % myCaseTypes, "userid":"%i" % myUser.userID}
        else:
            myCause = {"archived":"%i" % myCaseTypes}

        quCases = kumula.DB.doSelect("cases", ("caseid","reference","title","userid","casetype","date_creation"), "caseid", myCause)
        while quCases.next():
            myID    = quCases.value(0).toInt()
            myIntID = quCases.value(1).toString()
            myTitle = quCases.value(2).toString()
            myUser  = getUserByID(quCases.value(3).toInt()).getFormattedName()
            myType  = self.caseTypes[quCases.value(4).toInt()]
            myDate  = quCases.value(5).toDate()
            item = KListViewItemWithID(myCaseList, myID)
            item.setText(0, myIntID)
            item.setText(1, myTitle)
            item.setText(2, myUser)
            item.setText(3, myType)
            item.setText(4, myDate.toString(Qt.LocalDate))
            if (myID == selectedCase):
                myCaseList.setSelected(item, 1)
                myCaseList.setCurrentItem(item)
                hasSelected = 1

        if (hasSelected == 0):
            # always select at least one case
            if myCaseList.firstChild():
                myCaseList.setCurrentItem(myCaseList.firstChild())
                myCaseList.setSelected(myCaseList.firstChild(), 1)
            else:
                # ...no cases here...
                pass



    def slotCaseListDoubleClick(self, aItem, aPoint, aColumn):
        print "slotCaseListDoubleClick was called."
        if (aItem):
            self.handleCaseOpen()


    def handleCaseNew(self):
        executed = 0
        myCaseTypes = kumula.DB.doCount("cases_casetypes")
        if (myCaseTypes > 0):
            myRecordTypes = kumula.DB.doCount("cases_recordtypes")
            if (myRecordTypes > 0):
                # Starting CaseData without parameters (=NEW case)
                kumula.runKumulaApp("casedata")
                executed = 1
        if (executed == 0):
            QMessageBox.information(self, "Attention", "You should enter some case types and record types,\nbefore you create a new case.\n\n Go to the 'Settings' menu to do so.")


    def handleCaseOpen(self):
        # Starting CaseData with selected case
        myCaseTypes = self.folderView.currentButton()
        if (myCaseTypes == 0):
            myCaseList = self.caseList
        else:
            myCaseList = self.caseArchieve
        if myCaseList.currentItem():
            myID = myCaseList.currentItem().tag
            print "Starting CASEDATA with case", myID
            kumula.runKumulaApp("casedata", {"case":"%i" % myID})


    def handleCaseActivate(self):
        if not self.caseArchieve.currentItem():
            return
        myMsg = QString("Are you sure to re-activate the case<br><b>")
        myMsg.append(self.caseArchieve.currentItem().text(1))
        myMsg.append("</b>?<br><br>If the case is currently opened read-only,<br>you have to re-open it to make any changes.")
        result = QMessageBox.question(self, "Activating case again", myMsg, "&Activate this case", "&Cancel")
        if (result == 0):
            kumula.DB.doUpdate("cases", {"archived":"0"}, {"caseid":"%i" % self.caseArchieve.currentItem().tag})
            self.updateCaseList()


    def handleCaseClose(self):
        if not self.caseList.currentItem():
            return
        myMsg = QString("You're about closing the case<br><b>")
        myMsg.append(self.caseList.currentItem().text(1))
        myMsg.append("</b>.<br><br>This means, you can't change anything<br>anymore! To re-activate the case,<br>choose 'Activate again' from the menu.")
        result = QMessageBox.information(self, "Closing case", myMsg, "Clo&se this case", "&Cancel")
        if (result == 0):
            kumula.DB.doUpdate("cases", {"archived":"1","date_archive":QDate.currentDate().toString(Qt.ISODate)}, {"caseid":"%i" % self.caseList.currentItem().tag})
            self.updateCaseList()


    def updateToggleButtons(self):
        self.actViewCasesAllUsers.setOn(self.showAllCases, 1)
        self.actViewEventsAllUsers.setOn(self.showAllEvents, 1)


    def handleViewCasesAllUsers(self):
        if (self.showAllCases == 0):
            self.showAllCases = 1
        else:
            self.showAllCases = 0
        kumula.ProgramSettings.writeInt("view", "showAllCases",  self.showAllCases)
        self.updateToggleButtons()
        self.handleViewRefresh()


    def handleViewEventsAllUsers(self):
        if (self.showAllEvents == 0):
            self.showAllEvents = 1
        else:
            self.showAllEvents = 0
        kumula.ProgramSettings.writeInt("view", "showAllEvents",  self.showAllEvents)
        self.updateToggleButtons()
        self.handleViewRefresh()


    def handleViewRefresh(self):
        self.updateView()


    def handleSettingsCaseTypes(self):
        dlg = KCaseTypeDialog(self)
        dlg.exec_loop()

    def handleSettingsRecordTypes(self):
        dlg = KRecordTypeDialog(self)
        dlg.exec_loop()

    def handleSettingsAutoID(self):
        from AutoIDDialogTmpl import AutoIDDialogTmpl
        dlg = AutoIDDialogTmpl(self)
        dlg.exec_loop()


    def handleExtrasRunClients(self):
        kumula.runKumulaApp("clients", {})

    def handleExtrasRunInstitutions(self):
        pass
#        kumula.runKumulaApp("institutions", {})

    def handleExtrasRunUsers(self):
        kumula.runKumulaApp("users", {})

    def handleExtrasRunCalc(self):
        kumula.runKumulaApp("bizcalc", {})



